@@ -46,8 +46,8 @@
 #include <geos/geom/util/SineStarFactory.h>
 #include <geos/util/GEOSException.h>
 #include <geos/util/IllegalArgumentException.h>
-#include <geos/opLinemerge.h>
-#include <geos/opPolygonize.h>
+#include <geos/operation/linemerge/LineMerger.h>
+#include <geos/operation/polygonize/Polygonizer.h>
 #include <geos/constants.h>
 #include <vector>
 #include <sstream>
@@ -443,7 +443,7 @@ do_all()
     geoms->push_back(create_rectangle(-5, -5, 10, 10)); // a square
     geoms->push_back(create_rectangle(-5, -5, 10, 20)); // a rectangle
     // The upper-right quarter of a vertical ellipse
-    geoms->push_back(create_arc(0, 0, 10, 20, 0, M_PI / 2));
+    geoms->push_back(create_arc(0, 0, 10, 20, 0, MATH_PI / 2));
     geoms->push_back(create_sinestar(10, 10, 100, 5, 2).release()); // a sine star
 #endif
 
@@ -1073,12 +1073,11 @@ do_all()
     /////////////////////////////////////////////
     LineMerger lm;
     lm.add(geoms);
-    vector<LineString*>* mls = lm.getMergedLineStrings();
+    auto mls = lm.getMergedLineStrings();
     newgeoms = new vector<const Geometry*>;
-    for(unsigned int i = 0; i < mls->size(); i++) {
-        newgeoms->push_back((*mls)[i]);
+    for(unsigned int i = 0; i < mls.size(); i++) {
+        newgeoms->push_back(mls[i].release());
     }
-    delete mls;
 
     cout << endl << "----- HERE IS THE LINEMERGE OUTPUT ------" << endl;
     wkt_print_geoms(newgeoms);
@@ -1100,8 +1099,8 @@ do_all()
     plgnzr.add(geoms);
     auto polys = plgnzr.getPolygons();
     newgeoms = new vector<const Geometry*>;
-    for(unsigned int i = 0; i < polys->size(); i++) {
-        newgeoms->push_back((*polys)[i].release());
+    for(unsigned int i = 0; i < polys.size(); i++) {
+        newgeoms->push_back(polys[i].release());
     }
 
     cout << endl << "----- HERE IS POLYGONIZE OUTPUT ------" << endl;
