@@ -3,6 +3,7 @@
  * GEOS - Geometry Engine Open Source
  * http://geos.osgeo.org
  *
+ * Copyright (C) 2020 Sandro Santilli <strk@kbt.io>
  * Copyright (C) 2006 Refractions Research Inc.
  *
  * This is free software; you can redistribute and/or modify it under
@@ -22,6 +23,7 @@
 
 #include <geos/geom/prep/BasicPreparedGeometry.h> // for inheritance
 #include <geos/noding/SegmentString.h>
+#include <geos/operation/distance/IndexedFacetDistance.h>
 
 #include <memory>
 
@@ -53,6 +55,7 @@ private:
     mutable std::unique_ptr<noding::FastSegmentSetIntersectionFinder> segIntFinder;
     mutable std::unique_ptr<algorithm::locate::PointOnGeometryLocator> ptOnGeomLoc;
     mutable noding::SegmentString::ConstVect segStrings;
+    mutable std::unique_ptr<operation::distance::IndexedFacetDistance> indexedDistance;
 
 protected:
 public:
@@ -61,11 +64,13 @@ public:
 
     noding::FastSegmentSetIntersectionFinder* getIntersectionFinder() const;
     algorithm::locate::PointOnGeometryLocator* getPointLocator() const;
+    operation::distance::IndexedFacetDistance* getIndexedFacetDistance() const;
 
     bool contains(const geom::Geometry* g) const override;
     bool containsProperly(const geom::Geometry* g) const override;
     bool covers(const geom::Geometry* g) const override;
     bool intersects(const geom::Geometry* g) const override;
+    double distance(const geom::Geometry* g) const override;
 
 };
 
