@@ -173,6 +173,12 @@ Envelope::expandToInclude(const Envelope* other)
     }
 }
 
+void
+Envelope::expandToInclude(const Envelope& other)
+{
+    return expandToInclude(&other);
+}
+
 /*public*/
 bool
 Envelope::covers(double x, double y) const
@@ -234,37 +240,6 @@ Envelope::toString() const
     return s.str();
 }
 
-/*public*/
-double
-Envelope::distance(const Envelope* env) const
-{
-    if(intersects(env)) {
-        return 0;
-    }
-    double dx = 0.0;
-    if(maxx < env->minx) {
-        dx = env->minx - maxx;
-    }
-    if(minx > env->maxx) {
-        dx = minx - env->maxx;
-    }
-    double dy = 0.0;
-    if(maxy < env->miny) {
-        dy = env->miny - maxy;
-    }
-    if(miny > env->maxy) {
-        dy = miny - env->maxy;
-    }
-    // if either is zero, the envelopes overlap either vertically or horizontally
-    if(dx == 0.0) {
-        return dy;
-    }
-    if(dy == 0.0) {
-        return dx;
-    }
-    return sqrt(dx * dx + dy * dy);
-}
-
 /*public*/
 bool
 operator==(const Envelope& a, const Envelope& b)
