@@ -41,17 +41,21 @@ using namespace std;
 namespace geos {
 namespace geom { // geos::geom
 
+const static FixedSizeCoordinateSequence<0> emptyCoords2d(2);
+const static FixedSizeCoordinateSequence<0> emptyCoords3d(3);
+
 
 /*protected*/
 Point::Point(CoordinateSequence* newCoords, const GeometryFactory* factory)
     :
     Geometry(factory),
-    empty(false)
+    empty2d(false),
+    empty3d(false)
 {
     std::unique_ptr<CoordinateSequence> coords(newCoords);
 
     if(coords == nullptr) {
-        empty = true;
+        empty2d = true;
         return;
     }
 
@@ -59,14 +63,17 @@ Point::Point(CoordinateSequence* newCoords, const GeometryFactory* factory)
         coordinates.setAt(coords->getAt(0), 0);
     } else if (coords->getSize() > 1) {
         throw util::IllegalArgumentException("Point coordinate list must contain a single element");
+    } else if (coords->getDimension() == 3) {
+        empty3d = true;
     } else {
-        empty = true;
+        empty2d = true;
     }
 }
 
 Point::Point(const Coordinate & c, const GeometryFactory* factory) :
     Geometry(factory),
-    empty(false)
+    empty2d(false),
+    empty3d(false)
 {
     coordinates.setAt(c, 0);
 }
@@ -76,26 +83,27 @@ Point::Point(const Point& p)
     :
     Geometry(p),
     coordinates(p.coordinates),
-    empty(p.empty)
+    empty2d(p.empty2d),
+    empty3d(p.empty3d)
 {
 }
 
 std::unique_ptr<CoordinateSequence>
 Point::getCoordinates() const
 {
-    return coordinates.clone();
+    return getCoordinatesRO()->clone();
 }
 
 size_t
 Point::getNumPoints() const
 {
-    return empty ? 0 : 1;
+    return isEmpty() ? 0 : 1;
 }
 
 bool
 Point::isEmpty() const
 {
-    return empty;
+    return empty2d || empty3d;
 }
 
 bool
@@ -110,10 +118,10 @@ Point::getDimension() const
     return Dimension::P; // point
 }
 
-int
+uint8_t
 Point::getCoordinateDimension() const
 {
-    return (int) coordinates.getDimension();
+    return (uint8_t) getCoordinatesRO()->getDimension();
 }
 
 int
@@ -152,7 +160,7 @@ Point::getZ() const
 const Coordinate*
 Point::getCoordinate() const
 {
-    return empty ? nullptr : &coordinates[0];
+    return isEmpty() ? nullptr : &coordinates[0];
 }
 
 string
@@ -191,6 +199,9 @@ Point::apply_ro(CoordinateFilter* filter) const
 void
 Point::apply_rw(const CoordinateFilter* filter)
 {
+    if (isEmpty()) {
+        return;
+    }
     coordinates.apply_rw(filter);
 }
 
@@ -284,6 +295,11 @@ Point::getGeometryTypeId() const
 const CoordinateSequence*
 Point::getCoordinatesRO() const
 {
+    if (empty2d) {
+        return &emptyCoords2d;
+    } else if (empty3d) {
+        return &emptyCoords3d;
+    }
     return &coordinates;
 }
 
