@@ -8,7 +8,7 @@
  *
  * This is free software; you can redistribute and/or modify it under
  * the terms of the GNU Lesser General Public Licence as published
- * by the Free Software Foundation. 
+ * by the Free Software Foundation.
  * See the COPYING file for more information.
  *
  **********************************************************************/
@@ -26,36 +26,37 @@ namespace geos {
 namespace index { // geos.index
 namespace strtree { // geos.index.strtree
 
-Interval::Interval(double newMin,double newMax)
+Interval::Interval(double newMin, double newMax)
 {
-	assert(newMin<=newMax);
-	imin=newMin;
-	imax=newMax;
+    assert(newMin <= newMax);
+    imin = newMin;
+    imax = newMax;
 }
 
 double
 Interval::getCentre()
 {
-	return (imin+imax)/2;
+    return (imin + imax) / 2;
 }
 
 Interval*
-Interval::expandToInclude(const Interval *other)
+Interval::expandToInclude(const Interval* other)
 {
-	imax=max(imax,other->imax);
-	imin=min(imin,other->imin);
-	return this;
+    imax = max(imax, other->imax);
+    imin = min(imin, other->imin);
+    return this;
 }
 
 bool
-Interval::intersects(const Interval *other) const
+Interval::intersects(const Interval* other) const
 {
-	return !(other->imin>imax || other->imax<imin);
+    return !(other->imin > imax || other->imax < imin);
 }
 
 bool
-Interval::equals(const Interval *other) const {
-	return imin==other->imin && imax==other->imax;
+Interval::equals(const Interval* other) const
+{
+    return imin == other->imin && imax == other->imax;
 }
 
 } // namespace geos.index.strtree
