@@ -27,58 +27,53 @@ namespace geos {
 namespace operation {
 namespace distance {
 
-STRtree*
+std::unique_ptr<STRtree>
 FacetSequenceTreeBuilder::build(const Geometry* g)
 {
-    std::unique_ptr<STRtree> tree(new STRtree(STR_TREE_NODE_CAPACITY));
-    std::unique_ptr<std::vector<FacetSequence*> > sections(computeFacetSequences(g));
-    for(std::vector<FacetSequence*>::iterator it = sections->begin(); it != sections->end(); ++it) {
-        FacetSequence* section = *it;
-        tree->insert(section->getEnvelope(), section);
-    }
+    auto tree = std::unique_ptr<STRtree>(new FacetSequenceTree(computeFacetSequences(g)));
 
     tree->build();
-    return tree.release();
+    return tree;
 }
 
-std::vector<FacetSequence*>*
+std::vector<FacetSequence>
 FacetSequenceTreeBuilder::computeFacetSequences(const Geometry* g)
 {
-    std::unique_ptr<std::vector<FacetSequence*> > sections(new std::vector<FacetSequence*>());
+    std::vector<FacetSequence> sections;
 
-    class FacetSequenceAdder;
     class FacetSequenceAdder : public geom::GeometryComponentFilter {
-        std::vector<FacetSequence*>*  m_sections;
+        std::vector<FacetSequence>&  m_sections;
 
     public :
-        FacetSequenceAdder(std::vector<FacetSequence*>* p_sections) :
+        FacetSequenceAdder(std::vector<FacetSequence> & p_sections) :
             m_sections(p_sections) {}
         void
         filter_ro(const Geometry* geom) override
         {
             if(const LineString* ls = dynamic_cast<const LineString*>(geom)) {
                 const CoordinateSequence* seq = ls->getCoordinatesRO();
-                addFacetSequences(geom, seq, *m_sections);
+                addFacetSequences(geom, seq, m_sections);
             }
             else if(const Point* pt = dynamic_cast<const Point*>(geom)) {
                 const CoordinateSequence* seq = pt->getCoordinatesRO();
-                addFacetSequences(geom, seq, *m_sections);
+                addFacetSequences(geom, seq, m_sections);
             }
         }
     };
 
-    FacetSequenceAdder facetSequenceAdder(sections.get());
+    FacetSequenceAdder facetSequenceAdder(sections);
     g->apply_ro(&facetSequenceAdder);
 
-    return sections.release();
+    return sections;
 }
 
 void
 FacetSequenceTreeBuilder::addFacetSequences(const Geometry* geom, const CoordinateSequence* pts,
-        std::vector<FacetSequence*>& sections)
+        std::vector<FacetSequence> & sections)
 {
     size_t i = 0;
     size_t size = pts->size();
+    if ( size == 0 ) return;
 
     while(i <= size - 1) {
         size_t end = i + FACET_SEQUENCE_SIZE + 1;
@@ -87,8 +82,7 @@ FacetSequenceTreeBuilder::addFacetSequences(const Geometry* geom, const Coordina
         if(end >= size - 1) {
             end = size;
         }
-        FacetSequence* sect = new FacetSequence(geom, pts, i, end);
-        sections.push_back(sect);
+        sections.emplace_back(geom, pts, i, end);
         i += FACET_SEQUENCE_SIZE;
     }
 }
