@@ -1,13 +1,8 @@
 //
 // Test Suite for C-API GEOSintersection
 
-#include <tut/tut.hpp>
-// geos
-#include <geos_c.h>
-// std
-#include <cstdarg>
-#include <cstdio>
-#include <cstdlib>
+#include "capi_test_utils.h"
+
 
 namespace tut {
 //
@@ -15,41 +10,25 @@ namespace tut {
 //
 
 // Common data used in test cases.
-struct test_capigeosintersection_data {
+struct test_capigeosintersection_data : public capitest::utility
+{
+
     GEOSWKTWriter* wktw_;
     GEOSGeometry* geom1_;
     GEOSGeometry* geom2_;
     GEOSGeometry* geom3_;
-
-    static void
-    notice(const char* fmt, ...)
-    {
-        std::fprintf(stdout, "NOTICE: ");
-
-        va_list ap;
-        va_start(ap, fmt);
-        std::vfprintf(stdout, fmt, ap);
-        va_end(ap);
-
-        std::fprintf(stdout, "\n");
-    }
+    GEOSGeometry* expected_;
 
     test_capigeosintersection_data()
         : geom1_(nullptr), geom2_(nullptr), geom3_(nullptr)
     {
-        initGEOS(notice, notice);
         wktw_ = GEOSWKTWriter_create();
         GEOSWKTWriter_setTrim(wktw_, 1);
         GEOSWKTWriter_setOutputDimension(wktw_, 3);
-    }
-
-    std::string
-    toWKT(GEOSGeometry* g)
-    {
-        char* wkt = GEOSWKTWriter_write(wktw_, g);
-        std::string ret(wkt);
-        GEOSFree(wkt);
-        return ret;
+        geom1_ = nullptr;
+        geom2_ = nullptr;
+        geom3_ = nullptr;
+        expected_ = nullptr;
     }
 
     ~test_capigeosintersection_data()
@@ -58,10 +37,11 @@ struct test_capigeosintersection_data {
         GEOSGeom_destroy(geom1_);
         GEOSGeom_destroy(geom2_);
         GEOSGeom_destroy(geom3_);
+        if (expected_) GEOSGeom_destroy(expected_);
         geom1_ = nullptr;
         geom2_ = nullptr;
         geom3_ = nullptr;
-        finishGEOS();
+        expected_ = nullptr;
     }
 
 };
@@ -88,7 +68,7 @@ void object::test<1>
 
     geom3_ = GEOSIntersection(geom1_, geom2_);
     ensure(nullptr != geom3_);
-    ensure_equals(toWKT(geom3_), std::string("GEOMETRYCOLLECTION EMPTY"));
+    ensure_geometry_equals(geom3_, "POLYGON EMPTY");
 }
 
 template<>
@@ -104,7 +84,7 @@ void object::test<2>
 
     geom3_ = GEOSIntersection(geom1_, geom2_);
     ensure(nullptr != geom3_);
-    ensure_equals(toWKT(geom3_), std::string("POINT (2 2)"));
+    ensure_geometry_equals(geom3_, "POINT (2 2)");
 }
 
 template<>
@@ -114,6 +94,7 @@ void object::test<3>
 {
     geom1_ = GEOSGeomFromWKT("MULTIPOLYGON(((0 0,0 10,10 10,10 0,0 0)))");
     geom2_ = GEOSGeomFromWKT("POLYGON((-1 1,-1 2,2 2,2 1,-1 1))");
+    expected_ = GEOSGeomFromWKT("POLYGON ((0 1, 0 2, 2 2, 2 1, 0 1))");
 
     ensure(nullptr != geom1_);
     ensure(nullptr != geom2_);
@@ -121,7 +102,7 @@ void object::test<3>
     geom3_ = GEOSIntersection(geom1_, geom2_);
 
     ensure(nullptr != geom3_);
-    ensure_equals(toWKT(geom3_), std::string("POLYGON ((0 1, 0 2, 2 2, 2 1, 0 1))"));
+    ensure_geometry_equals(geom3_, expected_, 0.1);
 }
 
 /* See http://trac.osgeo.org/geos/ticket/719 */
@@ -137,10 +118,13 @@ void object::test<4>
     ensure(nullptr != geom2_);
 
     geom3_ = GEOSIntersection(geom1_, geom2_);
+    GEOSNormalize(geom3_);
 
     ensure(nullptr != geom3_);
-    ensure_equals(toWKT(geom3_),
-                  std::string("GEOMETRYCOLLECTION (LINESTRING (1 2, 2 2), LINESTRING (2 1, 1 1), POLYGON ((0.5 1, 1 2, 1 1, 0.5 1)), POLYGON ((9 2, 9.5 1, 2 1, 2 2, 9 2)))"));
+
+    expected_ = GEOSGeomFromWKT("GEOMETRYCOLLECTION (LINESTRING (1 2, 2 2), LINESTRING (2 1, 1 1), POLYGON ((0.5 1, 1 2, 1 1, 0.5 1)), POLYGON ((9 2, 9.5 1, 2 1, 2 2, 9 2)))");
+    GEOSNormalize(expected_);
+    ensure(GEOSEqualsExact(expected_, geom3_, 0.00001));
 }
 
 } // namespace tut
