@@ -25,6 +25,7 @@
 #include <geos/geom/GeometryFactory.h>
 #include <geos/geom/prep/PreparedGeometry.h>
 #include <geos/io/WKTReader.h>
+#include <geos/io/WKTWriter.h>
 // std
 #include <memory>
 #include <cstdlib>
@@ -91,38 +92,63 @@ instanceOf(InstanceType const* instance)
     return dynamic_cast<Type const*>(instance);
 }
 
+inline void
+ensure_equals_xyz(geos::geom::Coordinate const& actual,
+                 geos::geom::Coordinate const& expected)
+{
+    ensure_equals("Coordinate X", actual.x, expected.x );
+    ensure_equals("Coordinate Y", actual.y, expected.y );
+    if ( std::isnan(expected.z) ) {
+        ensure("Coordinate Z should be NaN", std::isnan(actual.z) );
+    } else {
+        ensure_equals("Coordinate Z", actual.z, expected.z );
+    }
+}
+
+
+
 //
 // Geometries structure comparators
 //
 
+
+
+
+
 template <typename T1, typename T2>
 inline void
 ensure_equals_geometry(T1 const* lhs, T2 const* rhs)
 {
     assert(0 != lhs);
     assert(0 != rhs);
-
     assert(!"DIFFERENT TYPES ARE NOT OF THE SAME STRUCTURE");
+    ensure(lhs != 0 && rhs != 0 && lhs != rhs);
 }
 
 template <typename T>
 inline void
-ensure_equals_geometry(T const* lhs, T const* rhs)
+ensure_equals_geometry(T const* lhs_in, T const* rhs_in, double tolerance = 0.0)
 {
-    assert(nullptr != lhs);
-    assert(nullptr != rhs);
+    assert(nullptr != lhs_in);
+    assert(nullptr != rhs_in);
 
     using geos::geom::Polygon;
     using geos::geom::GeometryCollection;
 
+    // Take clones so we can normalize them
+    std::unique_ptr<geos::geom::Geometry> lhs = lhs_in->clone();
+    std::unique_ptr<geos::geom::Geometry> rhs = rhs_in->clone();
+    lhs->normalize();
+    rhs->normalize();
+
     ensure_equals("is-valid do not match",
                   lhs->isValid(), rhs->isValid());
 
     ensure_equals("is-empty do not match",
                   lhs->isEmpty(), rhs->isEmpty());
 
-    if(!isInstanceOf<GeometryCollection>(lhs)
-            && !isInstanceOf<GeometryCollection>(rhs)) {
+    if(!isInstanceOf<GeometryCollection>(lhs.get()) &&
+       !isInstanceOf<GeometryCollection>(rhs.get())) {
         ensure_equals("is-simple do not match",
                       lhs->isSimple(), rhs->isSimple());
     }
@@ -141,22 +167,30 @@ ensure_equals_geometry(T const* lhs, T const* rhs)
 
     // NOTE - mloskot: Intentionally disabled, so simplified geometry
     // can be compared to its original
-    //ensure_equals("number of points do not match",
-    //              lhs->getNumPoints(), rhs->getNumPoints());
+    ensure_equals("number of points do not match",
+                  lhs->getNumPoints(), rhs->getNumPoints());
 
+    bool areEqual = lhs->equalsExact(rhs.get(), tolerance);
+    if(!areEqual) {
+        std::cout << std::endl << rhs->toText() << std::endl << lhs->toText() << std::endl;
+    }
+
+    ensure("coordinates do not match", areEqual);
     // Dispatch to run more specific testes
-    if(isInstanceOf<Polygon>(lhs)
-            && isInstanceOf<Polygon>(rhs)) {
-        ensure_equals_geometry(instanceOf<Polygon>(lhs),
-                               instanceOf<Polygon>(rhs));
-    }
-    else if(isInstanceOf<GeometryCollection>(lhs)
-            && isInstanceOf<GeometryCollection>(rhs)) {
-        ensure_equals_geometry(instanceOf<GeometryCollection>(lhs),
-                               instanceOf<GeometryCollection>(rhs));
-    }
+    // if(isInstanceOf<Polygon>(lhs)
+    //         && isInstanceOf<Polygon>(rhs)) {
+    //     ensure_equals_geometry(instanceOf<Polygon>(lhs),
+    //                            instanceOf<Polygon>(rhs));
+    // }
+    // else if(isInstanceOf<GeometryCollection>(lhs)
+    //         && isInstanceOf<GeometryCollection>(rhs)) {
+    //     ensure_equals_geometry(instanceOf<GeometryCollection>(lhs),
+    //                            instanceOf<GeometryCollection>(rhs));
+    // }
 }
 
+
+
 template <>
 inline void
 ensure_equals_geometry(geos::geom::Polygon const* lhs,
@@ -201,6 +235,98 @@ ensure_equals_geometry(geos::geom::Geometry const* lhs,
     ensure_equals_geometry(lhs, &pg);
 }
 
+template <typename T>
+inline void
+ensure_equals_dims(T const *, T const *,
+                   unsigned int dims,
+                   double tolerance = 0.0);
+
+template <>
+inline void
+ensure_equals_dims(const geos::geom::CoordinateSequence *seq1,
+                   const geos::geom::CoordinateSequence *seq2,
+                   unsigned int dims, double tolerance)
+{
+    assert(nullptr != seq1);
+    assert(nullptr != seq2);
+
+    ensure_equals (seq1->size(), seq2->size());
+
+    ensure( seq1->getDimension() >= dims );
+    ensure( seq2->getDimension() >= dims );
+
+    for (unsigned int i = 0; i < seq1->size(); i++) {
+      for (unsigned int j = 0; j < dims; j++) {
+        double val1 = seq1->getOrdinate(i, j);
+        double val2 = seq2->getOrdinate(i, j);
+        if ( std::isnan(val1) )
+        {
+            ensure( std::isnan(val2) );
+        }
+        else
+        {
+            if ( tolerance > 0.0 )
+                ensure_distance( val1, val2, tolerance );
+            else
+                ensure_equals( val1, val2 );
+        }
+      }
+    }
+}
+
+template <typename T> inline void ensure_equals_exact_geometry_xyz(const T *lhs_in, const T *rhs_in, double tolerance = 0.0);
+
+template <>
+inline void
+ensure_equals_exact_geometry_xyz(const geos::geom::Geometry *lhs_in,
+                                 const geos::geom::Geometry *rhs_in,
+                                 double tolerance)
+{
+    assert(nullptr != lhs_in);
+    assert(nullptr != rhs_in);
+
+    using geos::geom::Point;
+    using geos::geom::LineString;
+    using geos::geom::Polygon;
+    using geos::geom::CoordinateSequence;
+    using geos::geom::GeometryCollection;
+
+    ensure_equals("type id do not match",
+                  lhs_in->getGeometryTypeId(), rhs_in->getGeometryTypeId());
+
+
+    if (const Point* gpt1 = dynamic_cast<const Point *>(lhs_in)) {
+      const Point *gpt2 = static_cast<const Point *>(rhs_in);
+      return ensure_equals_dims( gpt1->getCoordinatesRO(), gpt2->getCoordinatesRO(), 3, tolerance);
+    }
+    else if (const LineString* gln1 = dynamic_cast<const LineString *>(lhs_in)) {
+      const LineString *gln2 = static_cast<const LineString *>(rhs_in);
+      return ensure_equals_dims( gln1->getCoordinatesRO(), gln2->getCoordinatesRO(), 3, tolerance);
+    }
+    else if (dynamic_cast<const Polygon *>(lhs_in)) {
+      assert("Not implemented yet" == 0);
+    }
+    else if (const GeometryCollection* gc1 = dynamic_cast<const GeometryCollection *>(lhs_in)) {
+      const GeometryCollection *gc2 = static_cast<const GeometryCollection *>(rhs_in);
+      for (unsigned int i = 0; i < gc1->getNumGeometries(); i++) {
+        ensure_equals_exact_geometry_xyz(gc1->getGeometryN(i), gc2->getGeometryN(i), tolerance);
+      }
+    }
+}
+
+template <typename T>
+inline void
+ensure_equals_geometry_xyz(const T *lhs_in,
+                           const T *rhs_in,
+                           double tolerance=0.0)
+{
+    std::unique_ptr<geos::geom::Geometry> g1 = lhs_in->clone();
+    g1->normalize();
+    std::unique_ptr<geos::geom::Geometry> g2 = rhs_in->clone();
+    g2->normalize();
+    ensure_equals_exact_geometry_xyz(g1.get(), g2.get(), tolerance);
+}
+
 //
 // Utility functions
 //
